@@ -4,6 +4,7 @@ from typing import Sequence
 
 from codegen.models import DeferredVar, ImportHelper, PredefinedFn, Program, expr, stmt
 from loguru import logger
+from msgspec import convert
 
 from sera.misc import assert_not_null, to_snake_case
 from sera.models import App, DataCollection, Module, Package
@@ -21,7 +22,7 @@ def make_python_api(app: App, collections: Sequence[DataCollection]):
         route = app.api.pkg("routes").pkg(collection.get_pymodule_name())
 
         controllers = []
-        controllers.append(make_python_get_api(collection, route))
+        controllers.append(make_python_search_api(collection, route))
         controllers.append(make_python_get_by_id_api(collection, route))
         controllers.append(make_python_has_api(collection, route))
         controllers.append(make_python_create_api(collection, route))
@@ -57,6 +58,12 @@ def make_python_api(app: App, collections: Sequence[DataCollection]):
                                     ]
                                 ),
                             ),
+                            PredefinedFn.keyword_assignment(
+                                "tags",
+                                PredefinedFn.list(
+                                    [expr.ExprConstant(collection.get_pymodule_name())]
+                                ),
+                            ),
                         ],
                     ),
                 ),
@@ -106,7 +113,7 @@ def make_main(target_pkg: Package, routes: Sequence[Module]):
     outmod.write(program)
 
 
-def make_python_get_api(
+def make_python_search_api(
     collection: DataCollection, target_pkg: Package
 ) -> tuple[Module, str]:
     """Make an endpoint for querying resources"""
@@ -116,10 +123,8 @@ def make_python_get_api(
     import_helper = ImportHelper(program, GLOBAL_IDENTS)
 
     program.import_("__future__.annotations", True)
-    program.import_("typing.Annotated", True)
-    program.import_("litestar.get", True)
-    program.import_("litestar.Request", True)
-    program.import_("litestar.params.Parameter", True)
+    program.import_("litestar.post", True)
+    program.import_(app.config.path + ".schema", True)
     program.import_(app.config.path + ".API_DEBUG", True)
     program.import_(
         app.services.path
@@ -127,24 +132,43 @@ def make_python_get_api(
         True,
     )
     program.import_(
-        app.models.data.path + f".{collection.get_pymodule_name()}.{collection.name}",
+        app.models.path + ".data_schema.dataschema",
         True,
     )
-    program.import_("sera.libs.api_helper.parse_query", True)
+    program.import_("sera.libs.search_helper.Query", True)
 
-    func_name = "get_"
+    func_name = "search"
 
     program.root(
         stmt.LineBreak(),
-        lambda ast00: ast00.assign(
+        lambda ast: ast.assign(
             DeferredVar.simple("QUERYABLE_FIELDS"),
-            expr.ExprConstant(collection.get_queryable_fields()),
+            PredefinedFn.set(
+                [
+                    expr.ExprConstant(propname)
+                    for propname in collection.get_queryable_fields()
+                ]
+            ),
         ),
+        stmt.LineBreak(),
+        lambda ast: ast.assign(
+            DeferredVar.simple("JOIN_QUERYABLE_FIELDS"),
+            PredefinedFn.dict(
+                [
+                    (
+                        expr.ExprConstant(propname),
+                        PredefinedFn.set([expr.ExprConstant(f) for f in fields]),
+                    )
+                    for propname, fields in collection.get_join_queryable_fields().items()
+                ]
+            ),
+        ),
+        stmt.LineBreak(),
         stmt.PythonDecoratorStatement(
             expr.ExprFuncCall(
-                expr.ExprIdent("get"),
+                expr.ExprIdent("post"),
                 [
-                    expr.ExprConstant("/"),
+                    expr.ExprConstant("/q"),
                 ],
             )
         ),
@@ -152,44 +176,8 @@ def make_python_get_api(
             func_name,
             [
                 DeferredVar.simple(
-                    "limit",
-                    expr.ExprIdent(
-                        'Annotated[int, Parameter(default=10, description="The maximum number of records to return")]'
-                    ),
-                ),
-                DeferredVar.simple(
-                    "offset",
-                    type=expr.ExprIdent(
-                        'Annotated[int, Parameter(default=0, description="The number of records to skip before returning results")]'
-                    ),
-                ),
-                DeferredVar.simple(
-                    "unique",
-                    expr.ExprIdent(
-                        'Annotated[bool, Parameter(default=False, description="Whether to return unique results only")]'
-                    ),
-                ),
-                DeferredVar.simple(
-                    "sorted_by",
-                    expr.ExprIdent(
-                        "Annotated[list[str], Parameter(default=tuple(), description=\"list of field names to sort by, prefix a field with '-' to sort that field in descending order\")]"
-                    ),
-                ),
-                DeferredVar.simple(
-                    "group_by",
-                    expr.ExprIdent(
-                        'Annotated[list[str], Parameter(default=tuple(), description="list of field names to group by")]'
-                    ),
-                ),
-                DeferredVar.simple(
-                    "fields",
-                    expr.ExprIdent(
-                        'Annotated[list[str], Parameter(default=tuple(), description="list of field names to include in the results")]'
-                    ),
-                ),
-                DeferredVar.simple(
-                    "request",
-                    expr.ExprIdent("Request"),
+                    "data",
+                    expr.ExprIdent("Query"),
                 ),
                 DeferredVar.simple(
                     "session",
@@ -212,19 +200,27 @@ def make_python_get_api(
                     [],
                 ),
             ),
-            lambda ast101: ast101.assign(
-                DeferredVar.simple("query", expr.ExprIdent("ServiceQuery")),
+            stmt.SingleExprStatement(
                 expr.ExprFuncCall(
-                    expr.ExprIdent("parse_query"),
+                    PredefinedFn.attr_getter(
+                        expr.ExprIdent("data"),
+                        expr.ExprIdent("validate_and_normalize"),
+                    ),
                     [
-                        expr.ExprIdent("request"),
+                        PredefinedFn.item_getter(
+                            PredefinedFn.attr_getter(
+                                expr.ExprIdent("schema"), expr.ExprIdent("classes")
+                            ),
+                            expr.ExprConstant(collection.cls.name),
+                        ),
                         expr.ExprIdent("QUERYABLE_FIELDS"),
+                        expr.ExprIdent("JOIN_QUERYABLE_FIELDS"),
                         PredefinedFn.keyword_assignment(
                             "debug",
                             expr.ExprIdent("API_DEBUG"),
                         ),
                     ],
-                ),
+                )
             ),
             lambda ast102: ast102.assign(
                 DeferredVar.simple("result"),
@@ -232,67 +228,34 @@ def make_python_get_api(
                     expr.ExprFuncCall(
                         PredefinedFn.attr_getter(
                             expr.ExprIdent("service"),
-                            expr.ExprIdent("get"),
+                            expr.ExprIdent("search"),
                         ),
-                        [
-                            expr.ExprIdent("query"),
-                            PredefinedFn.keyword_assignment(
-                                "limit", expr.ExprIdent("limit")
-                            ),
-                            PredefinedFn.keyword_assignment(
-                                "offset", expr.ExprIdent("offset")
-                            ),
-                            PredefinedFn.keyword_assignment(
-                                "unique", expr.ExprIdent("unique")
-                            ),
-                            PredefinedFn.keyword_assignment(
-                                "sorted_by", expr.ExprIdent("sorted_by")
-                            ),
-                            PredefinedFn.keyword_assignment(
-                                "group_by", expr.ExprIdent("group_by")
-                            ),
-                            PredefinedFn.keyword_assignment(
-                                "fields", expr.ExprIdent("fields")
-                            ),
-                            PredefinedFn.keyword_assignment(
-                                "session", expr.ExprIdent("session")
-                            ),
-                        ],
+                        [expr.ExprIdent("data"), expr.ExprIdent("session")],
                     )
                 ),
             ),
             lambda ast103: ast103.return_(
-                PredefinedFn.dict(
+                expr.ExprFuncCall(
+                    PredefinedFn.attr_getter(
+                        expr.ExprIdent("data"),
+                        expr.ExprIdent("prepare_results"),
+                    ),
                     [
-                        (
+                        PredefinedFn.item_getter(
                             PredefinedFn.attr_getter(
-                                expr.ExprIdent(collection.name),
-                                expr.ExprIdent("__name__"),
-                            ),
-                            PredefinedFn.map_list(
-                                PredefinedFn.attr_getter(
-                                    expr.ExprIdent("result"), expr.ExprIdent("records")
-                                ),
-                                lambda item: expr.ExprMethodCall(
-                                    expr.ExprIdent(collection.name),
-                                    "from_db",
-                                    [item],
-                                ),
+                                expr.ExprIdent("schema"), expr.ExprIdent("classes")
                             ),
+                            expr.ExprConstant(collection.cls.name),
                         ),
-                        (
-                            expr.ExprConstant("total"),
-                            PredefinedFn.attr_getter(
-                                expr.ExprIdent("result"), expr.ExprIdent("total")
-                            ),
-                        ),
-                    ]
+                        expr.ExprIdent("dataschema"),
+                        expr.ExprIdent("result"),
+                    ],
                 )
             ),
         ),
     )
 
-    outmod = target_pkg.module("get")
+    outmod = target_pkg.module("search")
     outmod.write(program)
 
     return outmod, func_name
