@@ -12,6 +12,7 @@ from codegen.models import (
     stmt,
 )
 
+from sera.make.py_backend.misc import get_python_property_name
 from sera.misc import (
     assert_isinstance,
     assert_not_null,
@@ -22,6 +23,7 @@ from sera.models import (
     Cardinality,
     Class,
     DataProperty,
+    IndexType,
     ObjectProperty,
     Package,
     PyTypeWithDep,
@@ -109,21 +111,21 @@ def make_python_data_model(
             ), "If value_pass_as_args is provided, record should not be used as a dummy value should be passed instead."
         else:
             value = PredefinedFn.attr_getter(record, expr.ExprIdent(prop.name))
+
+        propname = get_python_property_name(prop)
         if isinstance(prop, ObjectProperty) and prop.target.db is not None:
             if prop.cardinality.is_star_to_many():
                 value = PredefinedFn.map_list(
                     value,
                     lambda item: PredefinedFn.attr_getter(
-                        item, expr.ExprIdent(prop.name + "_id")
+                        item, expr.ExprIdent(propname)
                     ),
                 )
             else:
                 assert (
                     value_pass_as_args is None
                 ), "Cannot use value_pass_as_args for a single object property."
-                value = PredefinedFn.attr_getter(
-                    record, expr.ExprIdent(prop.name + "_id")
-                )
+                value = PredefinedFn.attr_getter(record, expr.ExprIdent(propname))
 
             target_idprop = assert_not_null(prop.target.get_id_property())
             conversion_fn = get_data_conversion(
@@ -146,7 +148,9 @@ def make_python_data_model(
         mode: Literal["create", "update"],
         prop: DataProperty | ObjectProperty,
     ):
-        value = PredefinedFn.attr_getter(slf, expr.ExprIdent(prop.name))
+        propname = get_python_property_name(prop)
+
+        value = PredefinedFn.attr_getter(slf, expr.ExprIdent(propname))
         if isinstance(prop, ObjectProperty):
             if (
                 prop.target.db is not None
@@ -173,7 +177,7 @@ def make_python_data_model(
                         expr.ExprIdent(AssociationTable),
                         [
                             PredefinedFn.keyword_assignment(
-                                f"{prop.name}_id", conversion_fn(item)
+                                propname, conversion_fn(item)
                             )
                         ],
                     ),
@@ -231,6 +235,8 @@ def make_python_data_model(
             if prop.data.system_controlled is None:
                 continue
 
+            propname = get_python_property_name(prop)
+
             update_func = None
             if mode == "create":
                 if prop.data.system_controlled.on_create_bypass is not None:
@@ -289,7 +295,7 @@ def make_python_data_model(
 
             smt = stmt.AssignStatement(
                 PredefinedFn.attr_getter(
-                    expr.ExprIdent("self"), expr.ExprIdent(prop.name)
+                    expr.ExprIdent("self"), expr.ExprIdent(propname)
                 ),
                 epr,
             )
@@ -369,6 +375,7 @@ def make_python_data_model(
             ):
                 continue
 
+            propname = get_python_property_name(prop)
             if isinstance(prop, DataProperty):
                 pytype = prop.get_data_model_datatype().get_python_type().clone()
                 if len(prop.data.constraints) > 0:
@@ -414,9 +421,7 @@ def make_python_data_model(
                     )
 
                 cls_ast(
-                    stmt.DefClassVarStatement(
-                        prop.name, pytype.type, prop_default_value
-                    )
+                    stmt.DefClassVarStatement(propname, pytype.type, prop_default_value)
                 )
             elif isinstance(prop, ObjectProperty):
                 if prop.target.db is not None:
@@ -442,7 +447,7 @@ def make_python_data_model(
                 for dep in pytype.deps:
                     program.import_(dep, True)
 
-                cls_ast(stmt.DefClassVarStatement(prop.name, pytype.type))
+                cls_ast(stmt.DefClassVarStatement(propname, pytype.type))
 
         if is_on_create_value_updated:
             program.import_("typing.Optional", True)
@@ -554,6 +559,8 @@ def make_python_data_model(
             ):
                 continue
 
+            propname = get_python_property_name(prop)
+
             if isinstance(prop, DataProperty):
                 pytype = prop.get_data_model_datatype().get_python_type().clone()
 
@@ -598,9 +605,7 @@ def make_python_data_model(
                     )
 
                 cls_ast(
-                    stmt.DefClassVarStatement(
-                        prop.name, pytype.type, prop_default_value
-                    )
+                    stmt.DefClassVarStatement(propname, pytype.type, prop_default_value)
                 )
             elif isinstance(prop, ObjectProperty):
                 if prop.target.db is not None:
@@ -610,6 +615,7 @@ def make_python_data_model(
                         .get_data_model_datatype()
                         .get_python_type()
                     )
+
                 else:
                     pytype = PyTypeWithDep(
                         f"Update{prop.target.name}",
@@ -626,7 +632,7 @@ def make_python_data_model(
                 for dep in pytype.deps:
                     program.import_(dep, True)
 
-                cls_ast(stmt.DefClassVarStatement(prop.name, pytype.type))
+                cls_ast(stmt.DefClassVarStatement(propname, pytype.type))
 
         if is_on_update_value_updated:
             program.import_("typing.Optional", True)
@@ -676,7 +682,9 @@ def make_python_data_model(
                                     [
                                         PredefinedFn.attr_getter(
                                             expr.ExprIdent("self"),
-                                            expr.ExprIdent(prop.name),
+                                            expr.ExprIdent(
+                                                get_python_property_name(prop)
+                                            ),
                                         )
                                     ],
                                 )
@@ -740,6 +748,7 @@ def make_python_data_model(
                 # skip private fields as this is for APIs exchange
                 continue
 
+            propname = get_python_property_name(prop)
             if isinstance(prop, DataProperty):
                 pytype = prop.get_data_model_datatype().get_python_type()
                 if prop.is_optional:
@@ -748,7 +757,7 @@ def make_python_data_model(
                 for dep in pytype.deps:
                     program.import_(dep, True)
 
-                cls_ast(stmt.DefClassVarStatement(prop.name, pytype.type))
+                cls_ast(stmt.DefClassVarStatement(propname, pytype.type))
             elif isinstance(prop, ObjectProperty):
                 if prop.target.db is not None:
                     pytype = (
@@ -772,7 +781,7 @@ def make_python_data_model(
                 for dep in pytype.deps:
                     program.import_(dep, True)
 
-                cls_ast(stmt.DefClassVarStatement(prop.name, pytype.type))
+                cls_ast(stmt.DefClassVarStatement(propname, pytype.type))
 
         cls_ast(
             stmt.LineBreak(),
@@ -811,12 +820,14 @@ def make_python_data_model(
             as_composite_args = [DeferredVar.simple("cls")]
             as_composite_null_condition = []
             for prop in cls.properties.values():
+                propname = get_python_property_name(prop)
+
                 assert (
                     not prop.data.is_private
-                ), f"Embedded classes should not have private properties: {cls.name}.{prop.name}"
-                as_composite_args.append(DeferredVar.simple(prop.name))
+                ), f"Embedded classes should not have private properties: {cls.name}.{propname}"
+                as_composite_args.append(DeferredVar.simple(propname))
                 as_composite_null_condition.append(
-                    expr.ExprIs(expr.ExprIdent(prop.name), expr.ExprConstant(None))
+                    expr.ExprIs(expr.ExprIdent(propname), expr.ExprConstant(None))
                 )
 
             # For simplicity, we assume that this embedded class can be used in a nullable field (check if all properties are None and return None).
@@ -843,7 +854,9 @@ def make_python_data_model(
                             expr.ExprIdent("cls"),
                             [
                                 from_db_type_conversion(
-                                    expr.ExprIdent(""), prop, expr.ExprIdent(prop.name)
+                                    expr.ExprIdent(""),
+                                    prop,
+                                    expr.ExprIdent(get_python_property_name(prop)),
                                 )
                                 for prop in cls.properties.values()
                             ],
@@ -872,6 +885,35 @@ def make_python_data_model(
                 ),
             )
 
+    def make_data_schema_export():
+        program = Program()
+        program.import_("__future__.annotations", True)
+
+        expose_vars = [expr.ExprConstant(cls.name) for cls in schema.classes.values()]
+        expose_vars.append(expr.ExprConstant("dataschema"))
+
+        output = []
+        for cls in schema.classes.values():
+            program.import_(
+                f"{target_pkg.path}.{cls.get_pymodule_name()}.{cls.name}",
+                True,
+            )
+            output.append((expr.ExprConstant(cls.name), expr.ExprIdent(cls.name)))
+
+        program.root(
+            stmt.LineBreak(),
+            lambda ast: ast.assign(
+                DeferredVar.simple("dataschema"), PredefinedFn.dict(output)
+            ),
+            stmt.LineBreak(),
+            lambda ast: ast.assign(
+                DeferredVar.simple("__all__"),
+                PredefinedFn.list(expose_vars),
+            ),
+        )
+
+        target_pkg.parent().module("data_schema").write(program)
+
     for cls in schema.topological_sort():
         if cls.name in reference_classes:
             continue
@@ -884,6 +926,8 @@ def make_python_data_model(
         make_normal(program, cls)
         target_pkg.module(cls.get_pymodule_name()).write(program)
 
+    make_data_schema_export()
+
 
 def make_python_relational_model(
     schema: Schema,
@@ -903,12 +947,6 @@ def make_python_relational_model(
     """
     app = target_pkg.app
 
-    def get_property_name(prop: DataProperty | ObjectProperty):
-        if isinstance(prop, ObjectProperty):
-            if prop.target.db is not None:
-                return f"{prop.name}_id"
-        return prop.name
-
     def make_base(custom_types: Sequence[ObjectProperty]):
         """Make a base class for our database."""
         program = Program()
@@ -1087,6 +1125,66 @@ def make_python_relational_model(
 
         target_pkg.module("base").write(program)
 
+    def make_db_schema_export():
+        program = Program()
+        program.import_("__future__.annotations", True)
+
+        expose_vars = [
+            expr.ExprConstant(cls.name)
+            for cls in schema.classes.values()
+            if cls.db is not None
+        ]
+        expose_vars.append(expr.ExprConstant("dbschema"))
+
+        for name in ["engine", "async_engine", "get_session", "get_async_session"]:
+            program.import_(f"{target_pkg.path}.base.{name}", True)
+            expose_vars.append(expr.ExprConstant(name))
+
+        output = []
+        for cls in schema.classes.values():
+            if cls.db is None:
+                continue
+            program.import_(
+                f"{target_pkg.path}.{cls.get_pymodule_name()}.{cls.name}",
+                True,
+            )
+            output.append((expr.ExprConstant(cls.name), expr.ExprIdent(cls.name)))
+
+            # if there is a MANY-TO-MANY relationship, we need to add an association table as well
+            for prop in cls.properties.values():
+                if (
+                    not isinstance(prop, ObjectProperty)
+                    or prop.target.db is None
+                    or prop.cardinality != Cardinality.MANY_TO_MANY
+                ):
+                    continue
+
+                program.import_(
+                    f"{target_pkg.path}.{to_snake_case(cls.name + prop.target.name)}.{cls.name}{prop.target.name}",
+                    True,
+                )
+                output.append(
+                    (
+                        expr.ExprConstant(f"{cls.name}{prop.target.name}"),
+                        expr.ExprIdent(f"{cls.name}{prop.target.name}"),
+                    )
+                )
+                expose_vars.append(expr.ExprConstant(f"{cls.name}{prop.target.name}"))
+
+        program.root(
+            stmt.LineBreak(),
+            lambda ast: ast.assign(
+                DeferredVar.simple("dbschema"), PredefinedFn.dict(output)
+            ),
+            stmt.LineBreak(),
+            lambda ast: ast.assign(
+                DeferredVar.simple("__all__"),
+                PredefinedFn.list(expose_vars),
+            ),
+        )
+
+        target_pkg.module("__init__").write(program)
+
     def make_orm(cls: Class):
         if cls.db is None or cls.name in reference_classes:
             # skip classes that are not stored in the database
@@ -1105,20 +1203,86 @@ def make_python_relational_model(
         )
 
         index_stmts = []
-        if len(cls.db.indices) > 0:
+
+        if len(cls.db.indices) > 0 or any(
+            isinstance(prop, DataProperty)
+            and prop.db is not None
+            and prop.db.is_indexed
+            and (
+                prop.db.index_type == IndexType.POSTGRES_FTS_SEVI
+                or prop.db.index_type == IndexType.POSTGRES_TRIGRAM
+            )
+            for prop in cls.properties.values()
+        ):
             program.import_("sqlalchemy.Index", True)
+
+            fts_index = []
+            for prop in cls.properties.values():
+                if (
+                    not isinstance(prop, DataProperty)
+                    or prop.db is None
+                    or not prop.db.is_indexed
+                ):
+                    continue
+                propname = get_python_property_name(prop)
+                if prop.db.index_type == IndexType.POSTGRES_FTS_SEVI:
+                    fts_index.append(
+                        expr.ExprFuncCall(
+                            expr.ExprIdent("Index"),
+                            [
+                                expr.ExprConstant(
+                                    f"ix_{cls.db.table_name}_{propname}_gin"
+                                ),
+                                expr.ExprFuncCall(
+                                    ident_manager.use("text"),
+                                    [
+                                        expr.ExprConstant(
+                                            f"to_tsvector('sevi', {propname})"
+                                        )
+                                    ],
+                                ),
+                                PredefinedFn.keyword_assignment(
+                                    "postgresql_using", expr.ExprConstant("gin")
+                                ),
+                            ],
+                        )
+                    )
+                if prop.db.index_type == IndexType.POSTGRES_TRIGRAM:
+                    fts_index.append(
+                        expr.ExprFuncCall(
+                            expr.ExprIdent("Index"),
+                            [
+                                expr.ExprConstant(
+                                    f"ix_{cls.db.table_name}_{propname}_gist"
+                                ),
+                                expr.ExprFuncCall(
+                                    expr.ExprIdent("text"),
+                                    [
+                                        expr.ExprConstant(
+                                            f"f_unaccent({propname}) gist_trgm_ops(siglen=256)"
+                                        )
+                                    ],
+                                ),
+                                PredefinedFn.keyword_assignment(
+                                    "postgresql_using", expr.ExprConstant("gist")
+                                ),
+                            ],
+                        )
+                    )
+
             index_stmts.append(
                 stmt.DefClassVarStatement(
                     "_table_args__",
                     None,
                     PredefinedFn.tuple(
-                        [
+                        fts_index
+                        + [
                             expr.ExprFuncCall(
                                 expr.ExprIdent("Index"),
                                 [expr.ExprConstant(index.name)]
                                 + [
                                     expr.ExprConstant(
-                                        get_property_name(cls.properties[prop])
+                                        get_python_property_name(cls.properties[prop])
                                     )
                                     for prop in index.columns
                                 ]
@@ -1215,7 +1379,8 @@ def make_python_relational_model(
                                 "unique", expr.ExprConstant(True)
                             )
                         )
-                    elif prop.db.is_indexed:
+                    elif prop.db.is_indexed and prop.db.index_type == IndexType.DEFAULT:
+                        # only add index=True for default index type
                         propvalargs.append(
                             PredefinedFn.keyword_assignment(
                                 "index", expr.ExprConstant(True)
@@ -1231,10 +1396,44 @@ def make_python_relational_model(
                     expr.ExprIdent("mapped_column"), propvalargs
                 )
                 cls_ast(stmt.DefClassVarStatement(propname, proptype, propval))
+
+                if prop.db.foreign_key is not None:
+                    # add a relationship property for foreign key primary key so that we can do eager join in SQLAlchemy
+                    program.import_("sqlalchemy.orm.relationship", True)
+                    if prop.db.foreign_key.name != cls.name:
+                        ident_manager.python_import_for_hint(
+                            target_pkg.path
+                            + f".{prop.db.foreign_key.get_pymodule_name()}.{prop.db.foreign_key.name}",
+                            True,
+                        )
+                    cls_ast(
+                        stmt.DefClassVarStatement(
+                            propname + "_relobj",
+                            f"Mapped[{prop.db.foreign_key.name}]",
+                            expr.ExprFuncCall(
+                                expr.ExprIdent("relationship"),
+                                [
+                                    PredefinedFn.keyword_assignment(
+                                        "lazy",
+                                        expr.ExprConstant("raise_on_sql"),
+                                    ),
+                                    PredefinedFn.keyword_assignment(
+                                        "foreign_keys",
+                                        expr.ExprIdent(propname),
+                                    ),
+                                    PredefinedFn.keyword_assignment(
+                                        "init",
+                                        expr.ExprConstant(False),
+                                    ),
+                                ],
+                            ),
+                        )
+                    )
             else:
                 assert isinstance(prop, ObjectProperty)
                 make_python_relational_object_property(
                     program=program,
+                    ident_manager=ident_manager,
                     target_pkg=target_pkg,
                     target_data_pkg=target_data_pkg,
                     cls_ast=cls_ast,
@@ -1256,9 +1455,13 @@ def make_python_relational_model(
     )
     make_base(custom_types)
 
+    # export the db classes in the __init__ file
+    make_db_schema_export()
+
 
 def make_python_relational_object_property(
     program: Program,
+    ident_manager: ImportHelper,
     target_pkg: Package,
     target_data_pkg: Package,
     cls_ast: AST,
@@ -1280,11 +1483,23 @@ def make_python_relational_object_property(
         if prop.cardinality.is_star_to_many():
             raise NotImplementedError((cls.name, prop.name))
 
+        program.import_("sqlalchemy.orm.relationship", True)
+        if prop.target.name != cls.name:
+            ident_manager.python_import_for_hint(
+                target_pkg.path
+                + f".{prop.target.get_pymodule_name()}.{prop.target.name}",
+                True,
+            )
+
         # we store this class in the database
-        propname = f"{prop.name}_id"
+        propname = get_python_property_name(prop)
         idprop = prop.target.get_id_property()
         assert idprop is not None
         idprop_pytype = idprop.datatype.get_sqlalchemy_type()
+
+        if prop.is_optional:
+            idprop_pytype = idprop_pytype.as_optional_type()
+
         for dep in idprop_pytype.deps:
             program.import_(dep, True)
 
@@ -1314,7 +1529,30 @@ def make_python_relational_object_property(
             ],
         )
 
-        cls_ast(stmt.DefClassVarStatement(propname, proptype, propval))
+        cls_ast(
+            stmt.DefClassVarStatement(propname, proptype, propval),
+            stmt.DefClassVarStatement(
+                prop.name,
+                f"Mapped[{prop.target.name}]",
+                expr.ExprFuncCall(
+                    expr.ExprIdent("relationship"),
+                    [
+                        PredefinedFn.keyword_assignment(
+                            "lazy",
+                            expr.ExprConstant("raise_on_sql"),
+                        ),
+                        PredefinedFn.keyword_assignment(
+                            "foreign_keys",
+                            expr.ExprIdent(propname),
+                        ),
+                        PredefinedFn.keyword_assignment(
+                            "init",
+                            expr.ExprConstant(False),
+                        ),
+                    ],
+                ),
+            ),
+        )
         return
 
     # if the target class is not in the database,
@@ -1337,16 +1575,16 @@ def make_python_relational_object_property(
         program.import_("sqlalchemy.orm.composite", True)
         propvalargs: list[expr.Expr] = [expr.ExprIdent(prop.target.name)]
         for p in prop.target.properties.values():
+            pdtype = assert_isinstance(p, DataProperty).datatype.get_sqlalchemy_type()
+            for dep in pdtype.deps:
+                program.import_(dep, True)
+
             propvalargs.append(
                 expr.ExprFuncCall(
                     expr.ExprIdent("mapped_column"),
                     [
                         expr.ExprConstant(f"{prop.name}_{p.name}"),
-                        expr.ExprIdent(
-                            assert_isinstance(p, DataProperty)
-                            .datatype.get_sqlalchemy_type()
-                            .type
-                        ),
+                        expr.ExprIdent(pdtype.type),
                         PredefinedFn.keyword_assignment(
                             "nullable",
                             expr.ExprConstant(prop.is_optional or p.is_optional),
@@ -1376,7 +1614,11 @@ def make_python_relational_object_property(
 
 
 def make_python_relational_object_property_many_to_many(
-    program: Program, ast: AST, target_pkg: Package, cls: Class, prop: ObjectProperty
+    program: Program,
+    ast: AST,
+    target_pkg: Package,
+    cls: Class,
+    prop: ObjectProperty,
 ):
     assert cls.db is not None
     assert prop.db is not None and prop.target.db is not None
@@ -1400,16 +1642,19 @@ def make_python_relational_object_property_many_to_many(
     newprogram.import_("sqlalchemy.orm.Mapped", True)
     newprogram.import_("sqlalchemy.orm.relationship", True)
     newprogram.import_(f"{target_pkg.path}.base.Base", True)
-    newprogram.import_("typing.TYPE_CHECKING", True)
-    newprogram.import_area.if_(expr.ExprIdent("TYPE_CHECKING"))(
-        lambda ast00: ast00.import_(
-            target_pkg.path + f".{cls.get_pymodule_name()}.{cls.name}",
-            is_import_attr=True,
-        ),
-        lambda ast10: ast10.import_(
-            target_pkg.path + f".{prop.target.get_pymodule_name()}.{prop.target.name}",
-            is_import_attr=True,
-        ),
+
+    ident_manager = ImportHelper(
+        newprogram,
+        GLOBAL_IDENTS,
+    )
+
+    ident_manager.python_import_for_hint(
+        target_pkg.path + f".{cls.get_pymodule_name()}.{cls.name}",
+        is_import_attr=True,
+    )
+    ident_manager.python_import_for_hint(
+        target_pkg.path + f".{prop.target.get_pymodule_name()}.{prop.target.name}",
+        is_import_attr=True,
     )
 
     newprogram.root(
