@@ -5,7 +5,13 @@ from sera.models._enum import Enum
 from sera.models._module import App, Module, Package
 from sera.models._multi_lingual_string import MultiLingualString
 from sera.models._parse import parse_schema
-from sera.models._property import Cardinality, DataProperty, ObjectProperty, Property
+from sera.models._property import (
+    Cardinality,
+    DataProperty,
+    IndexType,
+    ObjectProperty,
+    Property,
+)
 from sera.models._schema import Schema
 
 __all__ = [
@@ -14,6 +20,7 @@ __all__ = [
     "Property",
     "DataProperty",
     "ObjectProperty",
+    "IndexType",
     "Class",
     "Cardinality",
     "DataType",
